@@ -22,22 +22,21 @@ TEST_URL = 'http://127.0.0.1:5000/v3.0'
 
 class TestV3Auth(testtools.TestCase):
     def test_password_user_domain(self):
-        kargs = {
-            'trust_id': common.TEST_TRUST_ID,
-            'project_id': common.TEST_PROJECT_ID,
-            'project_name': common.TEST_PROJECT_NAME,
-            'user_name': common.TEST_USER,
-            'user_id': common.TEST_USER_ID,
-            'user_domain_id': common.TEST_DOMAIN_ID,
-            'user_domain_name': common.TEST_DOMAIN_NAME,
-            'password': common.TEST_PASS,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'trust_id': common.TEST_TRUST_ID,
+                 'project_id': common.TEST_PROJECT_ID,
+                 'project_name': common.TEST_PROJECT_NAME}
+
+        method = v3.PasswordMethod(username=common.TEST_USER,
+                                   user_id=common.TEST_USER_ID,
+                                   user_domain_id=common.TEST_DOMAIN_ID,
+                                   user_domain_name=common.TEST_DOMAIN_NAME,
+                                   password=common.TEST_PASS)
+        sot = v3.Auth(TEST_URL, [method], **kargs)
 
         self.assertEqual(1, len(sot.auth_methods))
         auther = sot.auth_methods[0]
         self.assertEqual(common.TEST_USER_ID, auther.user_id)
-        self.assertEqual(common.TEST_USER, auther.user_name)
+        self.assertEqual(common.TEST_USER, auther.username)
         self.assertEqual(common.TEST_DOMAIN_ID, auther.user_domain_id)
         self.assertEqual(common.TEST_DOMAIN_NAME, auther.user_domain_name)
         self.assertEqual(common.TEST_PASS, auther.password)
@@ -52,24 +51,24 @@ class TestV3Auth(testtools.TestCase):
         self.assertEqual(None, sot.project_domain_id)
         self.assertEqual(None, sot.project_domain_name)
         self.assertEqual(TEST_URL + '/auth/tokens', sot.token_url)
+        self.assertTrue(sot.include_catalog)
 
     def test_password_domain(self):
-        kargs = {
-            'domain_id': common.TEST_DOMAIN_ID,
-            'domain_name': common.TEST_DOMAIN_NAME,
-            'trust_id': common.TEST_TRUST_ID,
-            'project_id': common.TEST_PROJECT_ID,
-            'project_name': common.TEST_PROJECT_NAME,
-            'user_name': common.TEST_USER,
-            'user_id': common.TEST_USER_ID,
-            'password': common.TEST_PASS,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'domain_id': common.TEST_DOMAIN_ID,
+                 'domain_name': common.TEST_DOMAIN_NAME,
+                 'trust_id': common.TEST_TRUST_ID,
+                 'project_id': common.TEST_PROJECT_ID,
+                 'project_name': common.TEST_PROJECT_NAME}
+
+        methods = [v3.PasswordMethod(username=common.TEST_USER,
+                                     user_id=common.TEST_USER_ID,
+                                     password=common.TEST_PASS)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
 
         self.assertEqual(1, len(sot.auth_methods))
         auther = sot.auth_methods[0]
         self.assertEqual(common.TEST_USER_ID, auther.user_id)
-        self.assertEqual(common.TEST_USER, auther.user_name)
+        self.assertEqual(common.TEST_USER, auther.username)
         self.assertEqual(None, auther.user_domain_id)
         self.assertEqual(None, auther.user_domain_name)
         self.assertEqual(common.TEST_PASS, auther.password)
@@ -84,17 +83,17 @@ class TestV3Auth(testtools.TestCase):
         self.assertEqual(None, sot.project_domain_id)
         self.assertEqual(None, sot.project_domain_name)
         self.assertEqual(TEST_URL + '/auth/tokens', sot.token_url)
+        self.assertTrue(sot.include_catalog)
 
     def test_token_project_domain(self):
-        kargs = {
-            'project_domain_id': common.TEST_DOMAIN_ID,
-            'project_domain_name': common.TEST_DOMAIN_NAME,
-            'trust_id': common.TEST_TRUST_ID,
-            'project_id': common.TEST_PROJECT_ID,
-            'project_name': common.TEST_PROJECT_NAME,
-            'token': common.TEST_TOKEN,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'project_domain_id': common.TEST_DOMAIN_ID,
+                 'project_domain_name': common.TEST_DOMAIN_NAME,
+                 'trust_id': common.TEST_TRUST_ID,
+                 'project_id': common.TEST_PROJECT_ID,
+                 'project_name': common.TEST_PROJECT_NAME}
+
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
 
         self.assertEqual(1, len(sot.auth_methods))
         auther = sot.auth_methods[0]
@@ -121,8 +120,8 @@ class TestV3Auth(testtools.TestCase):
         return transport
 
     def test_authorize_token(self):
-        kargs = {'token': common.TEST_TOKEN}
-        sot = v3.Auth(TEST_URL, **kargs)
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods)
         xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
 
         resp = sot.authorize(xport)
@@ -138,27 +137,29 @@ class TestV3Auth(testtools.TestCase):
         ecatalog['version'] = 'v3'
         self.assertEqual(ecatalog, resp._info)
 
-    def test_authorize_token_access_info(self):
-        ecatalog = common.TEST_RESPONSE_DICT_V3['token'].copy()
-        kargs = {
-            'access_info': ecatalog,
-            'token': common.TEST_TOKEN,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+    def test_authorize_token_include_catalog(self):
+        kargs = {'include_catalog': False}
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
 
         resp = sot.authorize(xport)
 
-        ecatalog['auth_token'] = common.TEST_TOKEN
+        eurl = TEST_URL + '/auth/tokens?nocatalog'
+        eheaders = {'Accept': 'application/json',
+                    'X-Auth-Token': common.TEST_TOKEN}
+        ejson = {'auth': {'identity': {'token': {'id': common.TEST_TOKEN},
+                          'methods': ['token']}}}
+        xport.post.assert_called_with(eurl, headers=eheaders, json=ejson)
+        ecatalog = common.TEST_RESPONSE_DICT_V3['token'].copy()
+        ecatalog['auth_token'] = common.TEST_SUBJECT
         ecatalog['version'] = 'v3'
         self.assertEqual(ecatalog, resp._info)
 
     def test_authorize_token_domain_id(self):
-        kargs = {
-            'domain_id': common.TEST_DOMAIN_ID,
-            'token': common.TEST_TOKEN,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'domain_id': common.TEST_DOMAIN_ID}
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
 
         resp = sot.authorize(xport)
@@ -176,11 +177,9 @@ class TestV3Auth(testtools.TestCase):
         self.assertEqual(ecatalog, resp._info)
 
     def test_authorize_token_domain_name(self):
-        kargs = {
-            'domain_name': common.TEST_DOMAIN_NAME,
-            'token': common.TEST_TOKEN,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'domain_name': common.TEST_DOMAIN_NAME}
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
 
         resp = sot.authorize(xport)
@@ -199,11 +198,9 @@ class TestV3Auth(testtools.TestCase):
         self.assertEqual(ecatalog, resp._info)
 
     def test_authorize_token_project_id(self):
-        kargs = {
-            'project_id': common.TEST_PROJECT_ID,
-            'token': common.TEST_TOKEN,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'project_id': common.TEST_PROJECT_ID}
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
 
         resp = sot.authorize(xport)
@@ -222,12 +219,10 @@ class TestV3Auth(testtools.TestCase):
         self.assertEqual(ecatalog, resp._info)
 
     def test_authorize_token_project_name(self):
-        kargs = {
-            'project_name': common.TEST_PROJECT_NAME,
-            'project_domain_id': common.TEST_DOMAIN_ID,
-            'token': common.TEST_TOKEN,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'project_name': common.TEST_PROJECT_NAME,
+                 'project_domain_id': common.TEST_DOMAIN_ID}
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
 
         resp = sot.authorize(xport)
@@ -248,12 +243,10 @@ class TestV3Auth(testtools.TestCase):
         self.assertEqual(ecatalog, resp._info)
 
     def test_authorize_token_project_name_domain_name(self):
-        kargs = {
-            'project_name': common.TEST_PROJECT_NAME,
-            'project_domain_name': common.TEST_DOMAIN_NAME,
-            'token': common.TEST_TOKEN,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'project_name': common.TEST_PROJECT_NAME,
+                 'project_domain_name': common.TEST_DOMAIN_NAME}
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
 
         resp = sot.authorize(xport)
@@ -274,11 +267,9 @@ class TestV3Auth(testtools.TestCase):
         self.assertEqual(ecatalog, resp._info)
 
     def test_authorize_token_trust_id(self):
-        kargs = {
-            'token': common.TEST_TOKEN,
-            'trust_id': common.TEST_TRUST_ID,
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
+        kargs = {'trust_id': common.TEST_TRUST_ID}
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
 
         resp = sot.authorize(xport)
@@ -296,74 +287,55 @@ class TestV3Auth(testtools.TestCase):
         ecatalog['version'] = 'v3'
         self.assertEqual(ecatalog, resp._info)
 
+    def test_authorize_multi_method(self):
+        methods = [v3.PasswordMethod(username=common.TEST_USER,
+                                     password=common.TEST_PASS),
+                   v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods)
+        xport = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
+
+        resp = sot.authorize(xport)
+
+        eurl = TEST_URL + '/auth/tokens'
+        eheaders = {'Accept': 'application/json',
+                    'X-Auth-Token': common.TEST_TOKEN}
+        up = {'password': common.TEST_PASS, 'name': common.TEST_USER}
+        ejson = {'auth': {'identity': {'token': {'id': common.TEST_TOKEN},
+                                       'password': {'user': up},
+                                       'methods': ['password', 'token']}}}
+        xport.post.assert_called_with(eurl, headers=eheaders, json=ejson)
+        ecatalog = common.TEST_RESPONSE_DICT_V3['token'].copy()
+        ecatalog['auth_token'] = common.TEST_SUBJECT
+        ecatalog['version'] = 'v3'
+        self.assertEqual(ecatalog, resp._info)
+
     def test_authorize_mutually_exclusive(self):
         x = self.create_mock_transport(common.TEST_RESPONSE_DICT_V3)
-        kargs = {'token': common.TEST_TOKEN}
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
 
-        sot = v3.Auth(TEST_URL, **kargs)
-        sot.domain_id = 'a'
-        sot.project_id = 'b'
+        kargs = {'domain_id': 'a',
+                 'project_id': 'b'}
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         self.assertRaises(exceptions.AuthorizationFailure, sot.authorize, x)
 
-        sot = v3.Auth(TEST_URL, **kargs)
-        sot.domain_name = 'a'
-        sot.project_name = 'b'
+        kargs = {'domain_name': 'a',
+                 'project_name': 'b'}
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         self.assertRaises(exceptions.AuthorizationFailure, sot.authorize, x)
 
-        sot = v3.Auth(TEST_URL, **kargs)
-        sot.domain_name = 'a'
-        sot.trust_id = 'b'
+        kargs = {'domain_name': 'a',
+                 'trust_id': 'b'}
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         self.assertRaises(exceptions.AuthorizationFailure, sot.authorize, x)
 
-        sot = v3.Auth(TEST_URL, **kargs)
-        sot.project_id = 'a'
-        sot.trust_id = 'b'
+        kargs = {'project_id': 'a',
+                 'trust_id': 'b'}
+        sot = v3.Auth(TEST_URL, methods, **kargs)
         self.assertRaises(exceptions.AuthorizationFailure, sot.authorize, x)
 
     def test_authorize_bad_response(self):
-        kargs = {'token': common.TEST_TOKEN}
-        sot = v3.Auth(TEST_URL, **kargs)
+        methods = [v3.TokenMethod(token=common.TEST_TOKEN)]
+        sot = v3.Auth(TEST_URL, methods)
         xport = self.create_mock_transport({})
 
         self.assertRaises(exceptions.InvalidResponse, sot.authorize, xport)
-
-    def test_invalidate(self):
-        kargs = {
-            'user_name': common.TEST_USER,
-            'password': common.TEST_PASS,
-            'token': common.TEST_TOKEN,
-            'access_info': {},
-        }
-        sot = v3.Auth(TEST_URL, **kargs)
-        self.assertEqual(1, len(sot.auth_methods))
-        auther = sot.auth_methods[0]
-        self.assertEqual(common.TEST_TOKEN, auther.token)
-
-        self.assertEqual(True, sot.invalidate())
-
-        self.assertEqual(None, sot.access_info)
-        self.assertEqual(1, len(sot.auth_methods))
-        auther = sot.auth_methods[0]
-        self.assertEqual(common.TEST_USER, auther.user_name)
-        self.assertEqual(common.TEST_PASS, auther.password)
-
-    def test_valid_options(self):
-        expected = [
-            'access_info',
-            'auth_url',
-            'domain_id',
-            'domain_name',
-            'password',
-            'project_domain_id',
-            'project_domain_name',
-            'project_id',
-            'project_name',
-            'reauthenticate',
-            'token',
-            'trust_id',
-            'user_domain_id',
-            'user_domain_name',
-            'user_id',
-            'user_name',
-        ]
-        self.assertEqual(expected, v3.Auth.valid_options)
